@@ -12,7 +12,11 @@
 #include <libfilezilla/iputils.hpp>
 #include <libfilezilla/local_filesys.hpp>
 
-#ifndef __WXMSW__
+#include <wx/string.h>
+
+#ifndef FZ_WINDOWS
+	#include <sys/stat.h>
+
 	#define mutex mutex_override // Sadly on some platforms system headers include conflicting names
 	#include <netdb.h>
 	#undef mutex
@@ -32,13 +36,6 @@ std::list<CControlSocket::t_lockInfo> CControlSocket::m_lockInfoList;
 COpData::COpData(Command op_Id)
 	: opId(op_Id)
 {
-	opState = 0;
-
-	pNextOpData = 0;
-
-	waitForAsyncRequest = false;
-
-	holdsLock = false;
 }
 
 COpData::~COpData()
@@ -51,16 +48,6 @@ CControlSocket::CControlSocket(CFileZillaEnginePrivate & engine)
 	, event_handler(engine.event_loop_)
 	, engine_(engine)
 {
-	m_pCurOpData = 0;
-	m_nOpState = 0;
-	m_pCurrentServer = 0;
-
-	m_pCSConv = 0;
-	m_useUTF8 = false;
-
-	m_closed = false;
-
-	m_invalidateCurrentPath = false;
 }
 
 CControlSocket::~CControlSocket()
@@ -70,7 +57,6 @@ CControlSocket::~CControlSocket()
 	DoClose();
 
 	delete m_pCSConv;
-	m_pCSConv = 0;
 }
 
 int CControlSocket::Disconnect()
@@ -83,55 +69,63 @@ int CControlSocket::Disconnect()
 
 Command CControlSocket::GetCurrentCommandId() const
 {
-	if (m_pCurOpData)
+	if (m_pCurOpData) {
 		return m_pCurOpData->opId;
+	}
 
 	return engine_.GetCurrentCommandId();
 }
 
 void CControlSocket::LogTransferResultMessage(int nErrorCode, CFileTransferOpData *pData)
 {
-	bool tmp;
+	bool tmp{};
 
 	CTransferStatus const status = engine_.transfer_status_.Get(tmp);
 	if (!status.empty() && (nErrorCode == FZ_REPLY_OK || status.madeProgress)) {
 		int elapsed = static_cast<int>((fz::datetime::now() - status.started).get_seconds());
-		if (elapsed <= 0)
+		if (elapsed <= 0) {
 			elapsed = 1;
-		wxString time = wxString::Format(
-			wxPLURAL("%d second", "%d seconds", elapsed),
-			elapsed);
+		}
+		std::wstring time = fz::sprintf(fztranslate("%d second", "%d seconds", elapsed), elapsed);
 
 		int64_t transferred = status.currentOffset - status.startOffset;
-		wxString size = CSizeFormatBase::Format(&engine_.GetOptions(), transferred, true);
+		std::wstring size = CSizeFormatBase::Format(&engine_.GetOptions(), transferred, true);
 
 		MessageType msgType = MessageType::Error;
-		wxString msg;
+		std::wstring msg;
 		if (nErrorCode == FZ_REPLY_OK) {
 			msgType = MessageType::Status;
 			msg = _("File transfer successful, transferred %s in %s");
 		}
-		else if ((nErrorCode & FZ_REPLY_CANCELED) == FZ_REPLY_CANCELED)
+		else if ((nErrorCode & FZ_REPLY_CANCELED) == FZ_REPLY_CANCELED) {
 			msg = _("File transfer aborted by user after transferring %s in %s");
-		else if ((nErrorCode & FZ_REPLY_CRITICALERROR) == FZ_REPLY_CRITICALERROR)
+		}
+		else if ((nErrorCode & FZ_REPLY_CRITICALERROR) == FZ_REPLY_CRITICALERROR) {
 			msg = _("Critical file transfer error after transferring %s in %s");
-		else
+		}
+		else {
 			msg = _("File transfer failed after transferring %s in %s");
+		}
 		LogMessage(msgType, msg, size, time);
 	}
 	else {
-		if ((nErrorCode & FZ_REPLY_CANCELED) == FZ_REPLY_CANCELED)
+		if ((nErrorCode & FZ_REPLY_CANCELED) == FZ_REPLY_CANCELED) {
 			LogMessage(MessageType::Error, _("File transfer aborted by user"));
-		else if (nErrorCode == FZ_REPLY_OK) {
-			if (pData->transferInitiated)
-				LogMessage(MessageType::Status, _("File transfer successful"));
-			else
-				LogMessage(MessageType::Status, _("File transfer skipped"));
 		}
-		else if ((nErrorCode & FZ_REPLY_CRITICALERROR) == FZ_REPLY_CRITICALERROR)
+		else if (nErrorCode == FZ_REPLY_OK) {
+			if (pData->transferInitiated) {
+				LogMessage(MessageType::Status, _("File transfer successful"));
+			}
+			else {
+				LogMessage(MessageType::Status, _("File transfer skipped"));
+			}
+		}
+		else if ((nErrorCode & FZ_REPLY_CRITICALERROR) == FZ_REPLY_CRITICALERROR) {
 			LogMessage(MessageType::Error, _("Critical file transfer error"));
-		else
+		}
+		else {
 			LogMessage(MessageType::Error, _("File transfer failed"));
+		}
 	}
 }
 
@@ -143,8 +137,9 @@ int CControlSocket::ResetOperation(int nErrorCode)
 		LogMessage(MessageType::Debug_Warning, _T("ResetOperation with FZ_REPLY_WOULDBLOCK in nErrorCode (%d)"), nErrorCode);
 	}
 
-	if (m_pCurOpData && m_pCurOpData->holdsLock)
+	if (m_pCurOpData && m_pCurOpData->holdsLock) {
 		UnlockCache();
+	}
 
 	if (m_pCurOpData && m_pCurOpData->pNextOpData) {
 		COpData *pNext = m_pCurOpData->pNextOpData;
@@ -157,11 +152,12 @@ int CControlSocket::ResetOperation(int nErrorCode)
 		{
 			return ParseSubcommandResult(nErrorCode);
 		}
-		else
+		else {
 			return ResetOperation(nErrorCode);
+		}
 	}
 
-	wxString prefix;
+	std::wstring prefix;
 	if ((nErrorCode & FZ_REPLY_CRITICALERROR) == FZ_REPLY_CRITICALERROR &&
 		(!m_pCurOpData || m_pCurOpData->opId != Command::transfer))
 	{
@@ -173,21 +169,25 @@ int CControlSocket::ResetOperation(int nErrorCode)
 		switch (commandId)
 		{
 		case Command::none:
-			if( !prefix.empty() ) {
+			if (!prefix.empty()) {
 				LogMessage(MessageType::Error, _("Critical error"));
 			}
 			break;
 		case Command::connect:
-			if ((nErrorCode & FZ_REPLY_CANCELED) == FZ_REPLY_CANCELED)
+			if ((nErrorCode & FZ_REPLY_CANCELED) == FZ_REPLY_CANCELED) {
 				LogMessage(MessageType::Error, prefix + _("Connection attempt interrupted by user"));
-			else if (nErrorCode != FZ_REPLY_OK)
+			}
+			else if (nErrorCode != FZ_REPLY_OK) {
 				LogMessage(MessageType::Error, prefix + _("Could not connect to server"));
+			}
 			break;
 		case Command::list:
-			if ((nErrorCode & FZ_REPLY_CANCELED) == FZ_REPLY_CANCELED)
+			if ((nErrorCode & FZ_REPLY_CANCELED) == FZ_REPLY_CANCELED) {
 				LogMessage(MessageType::Error, prefix + _("Directory listing aborted by user"));
-			else if (nErrorCode != FZ_REPLY_OK)
+			}
+			else if (nErrorCode != FZ_REPLY_OK) {
 				LogMessage(MessageType::Error, prefix + _("Failed to retrieve directory listing"));
+			}
 			else {
 				if (m_CurrentPath.empty()) {
 					LogMessage(MessageType::Status, _("Directory listing successful"));
@@ -201,20 +201,23 @@ int CControlSocket::ResetOperation(int nErrorCode)
 			{
 				CFileTransferOpData *pData = static_cast<CFileTransferOpData *>(m_pCurOpData);
 				if (!pData->download && pData->transferInitiated) {
-					if (!m_pCurrentServer)
+					if (!m_pCurrentServer) {
 						LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Warning, _T("m_pCurrentServer is 0"));
+					}
 					else {
 						bool updated = engine_.GetDirectoryCache().UpdateFile(*m_pCurrentServer, pData->remotePath, pData->remoteFile, true, CDirectoryCache::file, (nErrorCode == FZ_REPLY_OK) ? pData->localFileSize : -1);
-						if (updated)
+						if (updated) {
 							engine_.SendDirectoryListingNotification(pData->remotePath, false, true, false);
+						}
 					}
 				}
 				LogTransferResultMessage(nErrorCode, pData);
 			}
 			break;
 		default:
-			if ((nErrorCode & FZ_REPLY_CANCELED) == FZ_REPLY_CANCELED)
+			if ((nErrorCode & FZ_REPLY_CANCELED) == FZ_REPLY_CANCELED) {
 				LogMessage(MessageType::Error, prefix + _("Interrupted by user"));
+			}
 			break;
 		}
 
@@ -238,7 +241,7 @@ int CControlSocket::DoClose(int nErrorCode /*=FZ_REPLY_DISCONNECTED*/)
 {
 	LogMessage(MessageType::Debug_Debug, _T("CControlSocket::DoClose(%d)"), nErrorCode);
 	if (m_closed) {
-		wxASSERT(!m_pCurOpData);
+		assert(!m_pCurOpData);
 		return nErrorCode;
 	}
 
@@ -252,24 +255,24 @@ int CControlSocket::DoClose(int nErrorCode /*=FZ_REPLY_DISCONNECTED*/)
 	return nErrorCode;
 }
 
-wxString CControlSocket::ConvertDomainName(wxString const& domain)
+std::wstring CControlSocket::ConvertDomainName(std::wstring const& domain)
 {
-#ifdef __WXMSW__
-	int len = IdnToAscii(IDN_ALLOW_UNASSIGNED, domain.wc_str(), domain.size() + 1, 0, 0);
+#ifdef FZ_WINDOWS
+	int len = IdnToAscii(IDN_ALLOW_UNASSIGNED, domain.c_str(), domain.size() + 1, 0, 0);
 	if (!len) {
 		LogMessage(MessageType::Debug_Warning, _T("Could not convert domain name"));
 		return domain;
 	}
 
 	wchar_t* output = new wchar_t[len];
-	int res = IdnToAscii(IDN_ALLOW_UNASSIGNED, domain.wc_str(), domain.size() + 1, output, len);
+	int res = IdnToAscii(IDN_ALLOW_UNASSIGNED, domain.c_str(), domain.size() + 1, output, len);
 	if (!res) {
 		delete [] output;
 		LogMessage(MessageType::Debug_Warning, _T("Could not convert domain name"));
 		return domain;
 	}
 
-	wxString ret(output);
+	std::wstring ret(output);
 	delete [] output;
 	return ret;
 #elif defined(AI_IDN)
@@ -283,7 +286,7 @@ wxString CControlSocket::ConvertDomainName(wxString const& domain)
 		return domain;
 	}
 
-	wxString result = wxConvCurrent->cMB2WX(output);
+	std::wstring result = fz::to_wstring(std::string(output));
 	idn_free(output);
 	return result;
 #endif
@@ -291,12 +294,13 @@ wxString CControlSocket::ConvertDomainName(wxString const& domain)
 
 void CControlSocket::Cancel()
 {
-	if (GetCurrentCommandId() != Command::none)
-	{
-		if (GetCurrentCommandId() == Command::connect)
+	if (GetCurrentCommandId() != Command::none) {
+		if (GetCurrentCommandId() == Command::connect) {
 			DoClose(FZ_REPLY_CANCELED);
-		else
+		}
+		else {
 			ResetOperation(FZ_REPLY_CANCELED);
+		}
 	}
 }
 
@@ -305,43 +309,48 @@ const CServer* CControlSocket::GetCurrentServer() const
 	return m_pCurrentServer;
 }
 
-bool CControlSocket::ParsePwdReply(wxString reply, bool unquoted /*=false*/, const CServerPath& defaultPath /*=CServerPath()*/)
+bool CControlSocket::ParsePwdReply(std::wstring reply, bool unquoted, CServerPath const& defaultPath)
 {
-	if (!unquoted)
-	{
-		int pos1 = reply.Find('"');
-		int pos2 = reply.Find('"', true);
-		if (pos1 == -1 || pos1 >= pos2) {
-			pos1 = reply.Find('\'');
-			pos2 = reply.Find('\'', true);
+	if (!unquoted) {
+		size_t pos1 = reply.find('"');
+		size_t pos2 = reply.rfind('"');
+		// Due to searching the same character, pos1 is npos iff pos2 is npos
 
-			if (pos1 != -1 && pos1 < pos2)
+		if (pos1 == std::wstring::npos || pos1 >= pos2) {
+			pos1 = reply.find('\'');
+			pos2 = reply.rfind('\'');
+
+			if (pos1 != std::wstring::npos && pos1 < pos2) {
 				LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Info, _T("Broken server sending single-quoted path instead of double-quoted path."));
-		}
-		if (pos1 == -1 || pos1 >= pos2) {
-			LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Info, _T("Broken server, no quoted path found in pwd reply, trying first token as path"));
-			pos1 = reply.Find(' ');
-			if (pos1 != -1) {
-				reply = reply.Mid(pos1 + 1);
-				pos2 = reply.Find(' ');
-				if (pos2 != -1)
-					reply = reply.Left(pos2);
 			}
-			else
+		}
+		if (pos1 == std::wstring::npos || pos1 >= pos2) {
+			LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Info, _T("Broken server, no quoted path found in pwd reply, trying first token as path"));
+			pos1 = reply.find(' ');
+			if (pos1 != std::wstring::npos) {
+				reply = reply.substr(pos1 + 1);
+				pos2 = reply.find(' ');
+				if (pos2 != std::wstring::npos)
+					reply = reply.substr(0, pos2);
+			}
+			else {
 				reply.clear();
+			}
 		}
 		else {
-			reply = reply.Mid(pos1 + 1, pos2 - pos1 - 1);
-			reply.Replace(_T("\"\""), _T("\""));
+			reply = reply.substr(pos1 + 1, pos2 - pos1 - 1);
+			fz::replace_substrings(reply, L"\"\"", L"\"");
 		}
 	}
 
 	m_CurrentPath.SetType(m_pCurrentServer->GetType());
 	if (reply.empty() || !m_CurrentPath.SetPath(reply)) {
-		if (reply.empty())
+		if (reply.empty()) {
 			LogMessage(MessageType::Error, _("Server returned empty path."));
-		else
+		}
+		else {
 			LogMessage(MessageType::Error, _("Failed to parse returned path."));
+		}
 
 		if (!defaultPath.empty()) {
 			LogMessage(MessageType::Debug_Warning, _T("Assuming path is '%s'."), defaultPath.GetPath());
@@ -356,8 +365,7 @@ bool CControlSocket::ParsePwdReply(wxString reply, bool unquoted /*=false*/, con
 
 int CControlSocket::CheckOverwriteFile()
 {
-	if (!m_pCurOpData)
-	{
+	if (!m_pCurOpData) {
 		LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Info, _T("Empty m_pCurOpData"));
 		ResetOperation(FZ_REPLY_INTERNALERROR);
 		return FZ_REPLY_ERROR;
@@ -375,10 +383,12 @@ int CControlSocket::CheckOverwriteFile()
 	bool dirDidExist;
 	bool matchedCase;
 	CServerPath remotePath;
-	if (pData->tryAbsolutePath || m_CurrentPath.empty())
+	if (pData->tryAbsolutePath || m_CurrentPath.empty()) {
 		remotePath = pData->remotePath;
-	else
+	}
+	else {
 		remotePath = m_CurrentPath;
+	}
 	bool found = engine_.GetDirectoryCache().LookupFile(entry, *m_pCurrentServer, remotePath, pData->remoteFile, dirDidExist, matchedCase);
 
 	// Ignore entries with wrong case
@@ -386,8 +396,9 @@ int CControlSocket::CheckOverwriteFile()
 		found = false;
 
 	if (!pData->download) {
-		if (!found && pData->remoteFileSize < 0 && pData->fileTime.empty())
+		if (!found && pData->remoteFileSize < 0 && pData->fileTime.empty()) {
 			return FZ_REPLY_OK;
+		}
 	}
 
 	CFileExistsNotification *pNotification = new CFileExistsNotification;
@@ -401,12 +412,15 @@ int CControlSocket::CheckOverwriteFile()
 	pNotification->remoteTime = pData->fileTime;
 	pNotification->ascii = !pData->transferSettings.binary;
 
-	if (pData->download && pNotification->localSize >= 0)
+	if (pData->download && pNotification->localSize >= 0) {
 		pNotification->canResume = true;
-	else if (!pData->download && pNotification->remoteSize >= 0)
+	}
+	else if (!pData->download && pNotification->remoteSize >= 0) {
 		pNotification->canResume = true;
-	else
+	}
+	else {
 		pNotification->canResume = false;
+	}
 
 	pNotification->localTime = fz::local_filesys::get_modification_time(fz::to_native(pData->localFile));
 
@@ -469,7 +483,7 @@ std::wstring CControlSocket::ConvToLocal(const char* buffer, size_t len)
 
 wchar_t* CControlSocket::ConvToLocalBuffer(const char* buffer, wxMBConv& conv, size_t len, size_t& outlen)
 {
-	wxASSERT(buffer && len > 0 && !buffer[len - 1]);
+	assert(buffer && len > 0 && !buffer[len - 1]);
 	outlen = conv.ToWChar(0, 0, buffer, len);
 	if (!outlen || outlen == wxCONV_FAILED)
 		return 0;
@@ -482,7 +496,7 @@ wchar_t* CControlSocket::ConvToLocalBuffer(const char* buffer, wxMBConv& conv, s
 wchar_t* CControlSocket::ConvToLocalBuffer(const char* buffer, size_t len, size_t& outlen)
 {
 	if (m_useUTF8) {
-#ifdef __WXMSW__
+#ifdef FZ_WINDOWS
 		// wxConvUTF8 is generic and slow.
 		// Use the highly optimized MultiByteToWideChar on Windows
 		// This helps when processing large directory listings.
@@ -518,25 +532,26 @@ wchar_t* CControlSocket::ConvToLocalBuffer(const char* buffer, size_t len, size_
 	return res;
 }
 
-wxCharBuffer CControlSocket::ConvToServer(const wxString& str, bool force_utf8)
+std::string CControlSocket::ConvToServer(std::wstring const& str, bool force_utf8)
 {
+	std::string ret;
 	if (m_useUTF8 || force_utf8) {
-		wxCharBuffer const buffer = str.utf8_str();
-		if (buffer || force_utf8)
-			return buffer;
+		ret = fz::to_utf8(str);
+		if (!ret.empty() || force_utf8) {
+			return ret;
+		}
 	}
 
 	if (m_pCSConv) {
-		wxCharBuffer const buffer = str.mb_str(*m_pCSConv);
-		if (buffer)
-			return buffer;
+		wxCharBuffer const buffer = wxString(str).mb_str(*m_pCSConv);
+		if (buffer) {
+			ret = buffer;
+			return ret;
+		}
 	}
 
-	wxCharBuffer buffer = str.mb_str(*wxConvCurrent);
-	if (!buffer)
-		buffer = str.To8BitData();
-
-	return buffer;
+	ret = fz::to_string(str);
+	return ret;
 }
 
 void CControlSocket::OnTimer(fz::timer_id)
@@ -549,7 +564,7 @@ void CControlSocket::OnTimer(fz::timer_id)
 
 		if ((!m_pCurOpData || !m_pCurOpData->waitForAsyncRequest) && !IsWaitingForLock()) {
 			if (elapsed > fz::duration::from_seconds(timeout)) {
-				LogMessage(MessageType::Error, wxPLURAL("Connection timed out after %d second of inactivity", "Connection timed out after %d seconds of inactivity", timeout), timeout);
+				LogMessage(MessageType::Error, fztranslate("Connection timed out after %d second of inactivity", "Connection timed out after %d seconds of inactivity", timeout), timeout);
 				DoClose(FZ_REPLY_TIMEOUT);
 				return;
 			}
@@ -611,8 +626,8 @@ const std::list<CControlSocket::t_lockInfo>::iterator CControlSocket::GetLockSta
 
 bool CControlSocket::TryLockCache(locking_reason reason, const CServerPath& directory)
 {
-	wxASSERT(m_pCurrentServer);
-	wxASSERT(m_pCurOpData);
+	assert(m_pCurrentServer);
+	assert(m_pCurOpData);
 
 	std::list<t_lockInfo>::iterator own = GetLockStatus();
 	if (own == m_lockInfoList.end())
@@ -637,8 +652,8 @@ bool CControlSocket::TryLockCache(locking_reason reason, const CServerPath& dire
 			}
 			return true;
 		}
-		wxASSERT(own->waiting);
-		wxASSERT(own->reason == reason);
+		assert(own->waiting);
+		assert(own->reason == reason);
 	}
 
 	// Needs to be set in any case so that ResetOperation
@@ -666,7 +681,7 @@ bool CControlSocket::TryLockCache(locking_reason reason, const CServerPath& dire
 
 bool CControlSocket::IsLocked(locking_reason reason, const CServerPath& directory)
 {
-	wxASSERT(m_pCurrentServer);
+	assert(m_pCurrentServer);
 
 	std::list<t_lockInfo>::iterator own = GetLockStatus();
 	if (own != m_lockInfoList.end())
@@ -699,11 +714,11 @@ void CControlSocket::UnlockCache()
 	if (iter == m_lockInfoList.end())
 		return;
 
-	wxASSERT(!iter->waiting || iter->lockcount == 0);
+	assert(!iter->waiting || iter->lockcount == 0);
 	if (!iter->waiting)
 	{
 		iter->lockcount--;
-		wxASSERT(iter->lockcount >= 0);
+		assert(iter->lockcount >= 0);
 		if (iter->lockcount)
 			return;
 	}
@@ -793,7 +808,7 @@ bool CControlSocket::IsWaitingForLock()
 
 void CControlSocket::InvalidateCurrentWorkingDir(const CServerPath& path)
 {
-	wxASSERT(!path.empty());
+	assert(!path.empty());
 	if (m_CurrentPath.empty())
 		return;
 
@@ -820,7 +835,7 @@ fz::duration CControlSocket::GetTimezoneOffset() const
 
 void CControlSocket::SendAsyncRequest(CAsyncRequestNotification* pNotification)
 {
-	wxASSERT(pNotification);
+	assert(pNotification);
 
 	pNotification->requestNumber = engine_.GetNextAsyncRequestNumber();
 
@@ -1015,12 +1030,11 @@ void CRealControlSocket::OnClose(int error)
 	DoClose();
 }
 
-int CRealControlSocket::Connect(const CServer &server)
+int CRealControlSocket::Connect(CServer const& server)
 {
 	SetWait(true);
 
-	if (server.GetEncodingType() == ENCODING_CUSTOM)
-	{
+	if (server.GetEncodingType() == ENCODING_CUSTOM) {
 		LogMessage(MessageType::Debug_Info, _T("Using custom encoding: %s"), server.GetCustomEncoding());
 		m_pCSConv = new wxCSConv(server.GetCustomEncoding());
 	}
@@ -1036,7 +1050,7 @@ int CRealControlSocket::Connect(const CServer &server)
 
 int CRealControlSocket::ContinueConnect()
 {
-	wxString host;
+	std::wstring host;
 	unsigned int port = 0;
 
 	const int proxy_type = engine_.GetOptions().GetOptionVal(OPTION_PROXY_TYPE);
@@ -1050,7 +1064,7 @@ int CRealControlSocket::ContinueConnect()
 		m_pProxyBackend = new CProxySocket(this, m_pSocket, this);
 		m_pBackend = m_pProxyBackend;
 		int res = m_pProxyBackend->Handshake(static_cast<CProxySocket::ProxyType>(proxy_type),
-											m_pCurrentServer->GetHost().ToStdWstring(), m_pCurrentServer->GetPort(),
+											m_pCurrentServer->GetHost(), m_pCurrentServer->GetPort(),
 											engine_.GetOptions().GetOption(OPTION_PROXY_USER),
 											engine_.GetOptions().GetOption(OPTION_PROXY_PASS));
 
@@ -1071,7 +1085,7 @@ int CRealControlSocket::ContinueConnect()
 			port = m_pCurrentServer->GetPort();
 		}
 	}
-	if (fz::get_address_type(host.ToStdWstring()) == fz::address_type::unknown) {
+	if (fz::get_address_type(host) == fz::address_type::unknown) {
 		LogMessage(MessageType::Status, _("Resolving address of %s"), host);
 	}
 
@@ -1116,7 +1130,7 @@ void CRealControlSocket::ResetSocket()
 
 bool CControlSocket::SetFileExistsAction(CFileExistsNotification *pFileExistsNotification)
 {
-	wxASSERT(pFileExistsNotification);
+	assert(pFileExistsNotification);
 
 	if (!m_pCurOpData || m_pCurOpData->opId != Command::transfer) {
 		LogMessage(__TFILE__, __LINE__, this, MessageType::Debug_Info, _T("No or invalid operation in progress, ignoring request reply %f"), pFileExistsNotification->GetRequestID());
@@ -1131,21 +1145,21 @@ bool CControlSocket::SetFileExistsAction(CFileExistsNotification *pFileExistsNot
 		SendNextCommand();
 		break;
 	case CFileExistsNotification::overwriteNewer:
-		if (pFileExistsNotification->localTime.empty() || pFileExistsNotification->remoteTime.empty())
+		if (pFileExistsNotification->localTime.empty() || pFileExistsNotification->remoteTime.empty()) {
 			SendNextCommand();
-		else if (pFileExistsNotification->download && pFileExistsNotification->localTime.earlier_than(pFileExistsNotification->remoteTime))
+		}
+		else if (pFileExistsNotification->download && pFileExistsNotification->localTime.earlier_than(pFileExistsNotification->remoteTime)) {
 			SendNextCommand();
-		else if (!pFileExistsNotification->download && pFileExistsNotification->localTime.later_than(pFileExistsNotification->remoteTime))
+		}
+		else if (!pFileExistsNotification->download && pFileExistsNotification->localTime.later_than(pFileExistsNotification->remoteTime)) {
 			SendNextCommand();
-		else
-		{
-			if (pData->download)
-			{
-				wxString filename = pData->remotePath.FormatFilename(pData->remoteFile);
+		}
+		else {
+			if (pData->download) {
+				std::wstring filename = pData->remotePath.FormatFilename(pData->remoteFile);
 				LogMessage(MessageType::Status, _("Skipping download of %s"), filename);
 			}
-			else
-			{
+			else {
 				LogMessage(MessageType::Status, _("Skipping upload of %s"), pData->localFile);
 			}
 			ResetOperation(FZ_REPLY_OK);
@@ -1154,11 +1168,12 @@ bool CControlSocket::SetFileExistsAction(CFileExistsNotification *pFileExistsNot
 	case CFileExistsNotification::overwriteSize:
 		/* First compare flags both size known but different, one size known and the other not (obviously they are different).
 		Second compare flags the remaining case in which we need to send command : both size unknown */
-		if ((pFileExistsNotification->localSize != pFileExistsNotification->remoteSize) || (pFileExistsNotification->localSize < 0))
+		if ((pFileExistsNotification->localSize != pFileExistsNotification->remoteSize) || (pFileExistsNotification->localSize < 0)) {
 			SendNextCommand();
+		}
 		else {
 			if (pData->download) {
-				wxString filename = pData->remotePath.FormatFilename(pData->remoteFile);
+				std::wstring filename = pData->remotePath.FormatFilename(pData->remoteFile);
 				LogMessage(MessageType::Status, _("Skipping download of %s"), filename);
 			}
 			else {
@@ -1168,41 +1183,44 @@ bool CControlSocket::SetFileExistsAction(CFileExistsNotification *pFileExistsNot
 		}
 		break;
 	case CFileExistsNotification::overwriteSizeOrNewer:
-		if (pFileExistsNotification->localTime.empty() || pFileExistsNotification->remoteTime.empty())
+		if (pFileExistsNotification->localTime.empty() || pFileExistsNotification->remoteTime.empty()) {
 			SendNextCommand();
+		}
 		/* First compare flags both size known but different, one size known and the other not (obviously they are different).
 		Second compare flags the remaining case in which we need to send command : both size unknown */
-		else if ((pFileExistsNotification->localSize != pFileExistsNotification->remoteSize) || (pFileExistsNotification->localSize < 0))
+		else if ((pFileExistsNotification->localSize != pFileExistsNotification->remoteSize) || (pFileExistsNotification->localSize < 0)) {
 			SendNextCommand();
-		else if (pFileExistsNotification->download && pFileExistsNotification->localTime.earlier_than(pFileExistsNotification->remoteTime))
+		}
+		else if (pFileExistsNotification->download && pFileExistsNotification->localTime.earlier_than(pFileExistsNotification->remoteTime)) {
 			SendNextCommand();
-		else if (!pFileExistsNotification->download && pFileExistsNotification->localTime.later_than(pFileExistsNotification->remoteTime))
+		}
+		else if (!pFileExistsNotification->download && pFileExistsNotification->localTime.later_than(pFileExistsNotification->remoteTime)) {
 			SendNextCommand();
-		else
-		{
-			if (pData->download)
-			{
-				wxString filename = pData->remotePath.FormatFilename(pData->remoteFile);
+		}
+		else {
+			if (pData->download) {
+				auto const filename = pData->remotePath.FormatFilename(pData->remoteFile);
 				LogMessage(MessageType::Status, _("Skipping download of %s"), filename);
 			}
-			else
-			{
+			else {
 				LogMessage(MessageType::Status, _("Skipping upload of %s"), pData->localFile);
 			}
 			ResetOperation(FZ_REPLY_OK);
 		}
 		break;
 	case CFileExistsNotification::resume:
-		if (pData->download && pData->localFileSize >= 0)
+		if (pData->download && pData->localFileSize >= 0) {
 			pData->resume = true;
-		else if (!pData->download && pData->remoteFileSize >= 0)
+		}
+		else if (!pData->download && pData->remoteFileSize >= 0) {
 			pData->resume = true;
+		}
 		SendNextCommand();
 		break;
 	case CFileExistsNotification::rename:
 		if (pData->download) {
 			{
-				wxString tmp;
+				std::wstring tmp;
 				CLocalPath l(pData->localFile, &tmp);
 				if (l.empty() || tmp.empty()) {
 					ResetOperation(FZ_REPLY_INTERNALERROR);
@@ -1258,13 +1276,11 @@ bool CControlSocket::SetFileExistsAction(CFileExistsNotification *pFileExistsNot
 		}
 		break;
 	case CFileExistsNotification::skip:
-		if (pData->download)
-		{
-			wxString filename = pData->remotePath.FormatFilename(pData->remoteFile);
+		if (pData->download) {
+			std::wstring filename = pData->remotePath.FormatFilename(pData->remoteFile);
 			LogMessage(MessageType::Status, _("Skipping download of %s"), filename);
 		}
-		else
-		{
+		else {
 			LogMessage(MessageType::Status, _("Skipping upload of %s"), pData->localFile);
 		}
 		ResetOperation(FZ_REPLY_OK);
@@ -1280,53 +1296,24 @@ bool CControlSocket::SetFileExistsAction(CFileExistsNotification *pFileExistsNot
 
 void CControlSocket::CreateLocalDir(std::wstring const & local_file)
 {
-	wxString file;
+	std::wstring file;
 	CLocalPath local_path(local_file, &file);
-	if (local_path.empty() || !local_path.HasParent())
+	if (local_path.empty() || !local_path.HasParent()) {
 		return;
-
-	// Only go back as far as needed. By comparison, wxWidgets'
-	// wxFileName::Mkdir always starts at the root.
-	std::vector<wxString> segments;
-	while (!local_path.Exists() && local_path.HasParent()) {
-		wxString segment;
-		local_path.MakeParent(&segment);
-		segments.push_back(segment);
 	}
 
 	CLocalPath last_successful;
-	for (auto iter = segments.rbegin(); iter != segments.rend(); ++iter) {
-		local_path.AddSegment(*iter);
+	local_path.Create(&last_successful);
 
-#ifdef FZ_WINDOWS
-		BOOL res = CreateDirectory(local_path.GetPath().c_str(), 0);
-		if (!res && GetLastError() != ERROR_ALREADY_EXISTS) {
-			break;
-		}
-#else
-		fz::native_string s = fz::to_native(local_path.GetPath());
-		if (s.empty()) {
-			break;
-		}
-
-		int res = mkdir(s.c_str(), 0777);
-		if (res && errno != EEXIST) {
-			break;
-		}
-#endif
-		last_successful = local_path;
+	if (!last_successful.empty()) {
+		// Send out notification
+		CLocalDirCreatedNotification *n = new CLocalDirCreatedNotification;
+		n->dir = last_successful;
+		engine_.AddNotification(n);
 	}
-
-	if (last_successful.empty())
-		return;
-
-	// Send out notification
-	CLocalDirCreatedNotification *n = new CLocalDirCreatedNotification;
-	n->dir = last_successful;
-	engine_.AddNotification(n);
 }
 
-int CControlSocket::List(CServerPath, wxString, int)
+int CControlSocket::List(CServerPath, std::wstring const&, int)
 {
 	return FZ_REPLY_NOTSUPPORTED;
 }
@@ -1338,7 +1325,7 @@ int CControlSocket::FileTransfer(std::wstring const&, CServerPath const&,
 	return FZ_REPLY_NOTSUPPORTED;
 }
 
-int CControlSocket::RawCommand(const wxString&)
+int CControlSocket::RawCommand(std::wstring const&)
 {
 	return FZ_REPLY_NOTSUPPORTED;
 }
@@ -1348,7 +1335,7 @@ int CControlSocket::Delete(const CServerPath&, std::deque<std::wstring>&&)
 	return FZ_REPLY_NOTSUPPORTED;
 }
 
-int CControlSocket::RemoveDir(const CServerPath&, const wxString&)
+int CControlSocket::RemoveDir(CServerPath const&, std::wstring const&)
 {
 	return FZ_REPLY_NOTSUPPORTED;
 }
