@@ -213,7 +213,7 @@ void CFileZillaEnginePrivate::ClearQueuedLogs(bool reset_flag)
 int CFileZillaEnginePrivate::ResetOperation(int nErrorCode)
 {
 	fz::scoped_lock lock(mutex_);
-	m_pLogging->LogMessage(MessageType::Debug_Debug, _T("CFileZillaEnginePrivate::ResetOperation(%d)"), nErrorCode);
+	m_pLogging->LogMessage(MessageType::Debug_Debug, L"CFileZillaEnginePrivate::ResetOperation(%d)", nErrorCode);
 
 	if (nErrorCode & FZ_REPLY_DISCONNECTED)
 		m_lastListDir.clear();
@@ -405,10 +405,10 @@ void CFileZillaEnginePrivate::SendDirectoryListingNotification(const CServerPath
 {
 	fz::scoped_lock lock(mutex_);
 
-	wxASSERT(m_pControlSocket);
+	assert(m_pControlSocket);
 
 	const CServer* const pOwnServer = m_pControlSocket->GetCurrentServer();
-	wxASSERT(pOwnServer);
+	assert(pOwnServer);
 
 	m_lastListDir = path;
 
@@ -488,18 +488,21 @@ unsigned int CFileZillaEnginePrivate::GetRemainingReconnectDelay(const CServer&
 			++iter;
 			m_failedLogins.erase(prev);
 		}
-		else if (!iter->critical && iter->server.GetHost() == server.GetHost() && iter->server.GetPort() == server.GetPort())
+		else if (!iter->critical && iter->server.GetHost() == server.GetHost() && iter->server.GetPort() == server.GetPort()) {
 			return delay * 1000 - span.get_milliseconds();
-		else if (iter->server == server)
+		}
+		else if (iter->server == server) {
 			return delay * 1000 - span.get_milliseconds();
-		else
+		}
+		else {
 			++iter;
+		}
 	}
 
 	return 0;
 }
 
-void CFileZillaEnginePrivate::OnTimer(int)
+void CFileZillaEnginePrivate::OnTimer(fz::timer_id)
 {
 	if (!m_retryTimer) {
 		return;
@@ -507,10 +510,11 @@ void CFileZillaEnginePrivate::OnTimer(int)
 	m_retryTimer = 0;
 
 	if (!m_pCurrentCommand || m_pCurrentCommand->GetId() != Command::connect) {
-		wxFAIL_MSG(_T("CFileZillaEnginePrivate::OnTimer called without pending Command::connect"));
+		m_pLogging->LogMessage(MessageType::Debug_Warning, L"CFileZillaEnginePrivate::OnTimer called without pending Command::connect");
 		return;
 	}
-	wxASSERT(!IsConnected());
+
+	assert(!IsConnected());
 
 	m_pControlSocket.reset();
 
@@ -522,7 +526,7 @@ int CFileZillaEnginePrivate::ContinueConnect()
 	fz::scoped_lock lock(mutex_);
 
 	if (!m_pCurrentCommand || m_pCurrentCommand->GetId() != Command::connect) {
-		m_pLogging->LogMessage(MessageType::Debug_Warning, _T("CFileZillaEnginePrivate::ContinueConnect called without pending Command::connect"));
+		m_pLogging->LogMessage(MessageType::Debug_Warning, L"CFileZillaEnginePrivate::ContinueConnect called without pending Command::connect");
 		return ResetOperation(FZ_REPLY_INTERNALERROR);
 	}
 
@@ -530,7 +534,7 @@ int CFileZillaEnginePrivate::ContinueConnect()
 	const CServer& server = pConnectCommand->GetServer();
 	unsigned int delay = GetRemainingReconnectDelay(server);
 	if (delay) {
-		m_pLogging->LogMessage(MessageType::Status, wxPLURAL("Delaying connection for %d second due to previously failed connection attempt...", "Delaying connection for %d seconds due to previously failed connection attempt...", (delay + 999) / 1000), (delay + 999) / 1000);
+		m_pLogging->LogMessage(MessageType::Status, fztranslate("Delaying connection for %d second due to previously failed connection attempt...", "Delaying connection for %d seconds due to previously failed connection attempt...", (delay + 999) / 1000), (delay + 999) / 1000);
 		stop_timer(m_retryTimer);
 		m_retryTimer = add_timer(fz::duration::from_milliseconds(delay), true);
 		return FZ_REPLY_WOULDBLOCK;
@@ -552,7 +556,7 @@ int CFileZillaEnginePrivate::ContinueConnect()
 		m_pControlSocket = std::make_unique<CHttpControlSocket>(*this);
 		break;
 	default:
-		m_pLogging->LogMessage(MessageType::Debug_Warning, _T("Not a valid protocol: %d"), server.GetProtocol());
+		m_pLogging->LogMessage(MessageType::Debug_Warning, L"Not a valid protocol: %d", server.GetProtocol());
 		return FZ_REPLY_SYNTAXERROR|FZ_REPLY_DISCONNECTED;
 	}
 
@@ -567,9 +571,9 @@ void CFileZillaEnginePrivate::InvalidateCurrentWorkingDirs(const CServerPath& pa
 {
 	fz::scoped_lock lock(mutex_);
 
-	wxASSERT(m_pControlSocket);
+	assert(m_pControlSocket);
 	const CServer* const pOwnServer = m_pControlSocket->GetCurrentServer();
-	wxASSERT(pOwnServer);
+	assert(pOwnServer);
 
 	for (auto & engine : m_engineList) {
 		if (!engine || engine == this)
@@ -680,7 +684,7 @@ void CFileZillaEnginePrivate::DoCancel()
 		return;
 
 	if (m_retryTimer) {
-		wxASSERT(m_pCurrentCommand && m_pCurrentCommand->GetId() == Command::connect);
+		assert(m_pCurrentCommand && m_pCurrentCommand->GetId() == Command::connect);
 
 		m_pControlSocket.reset();
 
@@ -818,7 +822,7 @@ int CFileZillaEnginePrivate::CacheLookup(const CServerPath& path, CDirectoryList
 	if (!IsConnected())
 		return FZ_REPLY_ERROR;
 
-	wxASSERT(m_pControlSocket->GetCurrentServer());
+	assert(m_pControlSocket->GetCurrentServer());
 
 	bool is_outdated = false;
 	if (!directory_cache_.Lookup(listing, *m_pControlSocket->GetCurrentServer(), path, true, is_outdated))
