@@ -18,7 +18,6 @@ enum interfaceOptions
 	OPTION_ASCIIFILES,
 	OPTION_ASCIINOEXT,
 	OPTION_ASCIIDOTFILE,
-	OPTION_THEME,
 	OPTION_LANGUAGE,
 	OPTION_LASTSERVERPATH,
 	OPTION_CONCURRENTDOWNLOADLIMIT,
@@ -62,7 +61,8 @@ enum interfaceOptions
 	OPTION_COMPARISONMODE,
 	OPTION_COMPARISON_THRESHOLD,
 	OPTION_SITEMANAGER_POSITION,
-	OPTION_THEME_ICONSIZE,
+	OPTION_ICONS_THEME,
+	OPTION_ICONS_SCALE,
 	OPTION_MESSAGELOG_TIMESTAMP,
 	OPTION_SITEMANAGER_LASTSELECTED,
 	OPTION_LOCALFILELIST_COLUMN_SHOWN,
@@ -105,6 +105,7 @@ enum interfaceOptions
 	OPTION_DEFAULT_SETTINGSDIR, // guaranteed to be (back)slash-terminated
 	OPTION_DEFAULT_KIOSKMODE,
 	OPTION_DEFAULT_DISABLEUPDATECHECK,
+	OPTION_DEFAULT_CACHE_DIR,
 
 	// Has to be last element
 	OPTIONS_NUM
@@ -146,6 +147,7 @@ public:
 	void SaveIfNeeded();
 
 	static CLocalPath GetUnadjustedSettingsDir();
+	CLocalPath GetCacheDirectory();
 
 protected:
 	COptions();
