@@ -11,12 +11,19 @@ DECLARE_EVENT_TYPE(fzEVT_GRANTEXCLUSIVEENGINEACCESS, -1)
 class CCommandQueue
 {
 public:
-	CCommandQueue(CFileZillaEngine *pEngine, CMainFrame* pMainFrame, CState* pState);
+	enum command_origin
+	{
+		any = -1,
+		normal, // Most user actions 
+		recursiveOperation
+	};
+
+	CCommandQueue(CFileZillaEngine *pEngine, CMainFrame* pMainFrame, CState& state);
 	~CCommandQueue();
 
-	void ProcessCommand(CCommand *pCommand);
+	void ProcessCommand(CCommand *pCommand, command_origin origin = normal);
 	void ProcessNextCommand();
-	bool Idle() const;
+	bool Idle(command_origin origin = any) const;
 	bool Cancel();
 	bool Quit();
 	void Finish(std::unique_ptr<COperationNotification> && pNotification);
@@ -27,6 +34,8 @@ public:
 	void ReleaseEngine();
 	bool EngineLocked() const { return m_exclusiveEngineLock; }
 
+	void ProcessDirectoryListing(CDirectoryListingNotification const& listingNotification);
+
 protected:
 	void ProcessReply(int nReplyCode, Command commandId);
 
@@ -34,7 +43,7 @@ protected:
 
 	CFileZillaEngine *m_pEngine;
 	CMainFrame* m_pMainFrame;
-	CState* m_pState;
+	CState& m_state;
 	bool m_exclusiveEngineRequest;
 	bool m_exclusiveEngineLock;
 	int m_requestId;
@@ -43,7 +52,18 @@ protected:
 	// Used to make this class reentrance-safe
 	int m_inside_commandqueue{};
 
-	std::deque<std::unique_ptr<CCommand>> m_CommandList;
+	struct CommandInfo {
+		CommandInfo() = default;
+		CommandInfo(command_origin o, std::unique_ptr<CCommand> && c)
+			: origin(o)
+			, command(std::move(c))
+		{}
+
+		command_origin origin;
+		std::unique_ptr<CCommand> command;
+		bool didReconnect{};
+	};
+	std::deque<CommandInfo> m_CommandList;
 
 	bool m_quit{};
 };
