@@ -4,6 +4,7 @@
 #include "filter.h"
 #include "state.h"
 
+class CActionAfterBlocker;
 class CQueueView;
 class CRecursiveOperation : public CStateEventHandler
 {
@@ -18,12 +19,12 @@ public:
 	enum OperationMode {
 		recursive_none,
 		recursive_transfer,
-		recursive_addtoqueue,
 		recursive_transfer_flatten,
-		recursive_addtoqueue_flatten,
 		recursive_delete,
 		recursive_chmod,
-		recursive_list
+		recursive_list,
+		recursive_synchronize_download,
+		recursive_synchronize_upload
 	};
 
 	bool IsActive() const { return GetOperationMode() != recursive_none; }
@@ -38,14 +39,18 @@ public:
 
 	void SetQueue(CQueueView* pQueue);
 
-	bool ChangeOperationMode(OperationMode mode);
+	void SetImmediate(bool immediate);
 
 protected:
+	bool m_immediate{true};
+
 	OperationMode m_operationMode;
 
 	CQueueView* m_pQueue{};
 
-	std::vector<CFilter> m_filters;
+	ActiveFilters m_filters;
+	
+	std::shared_ptr<CActionAfterBlocker> m_actionAfterBlocker;
 };
 
 #endif
\ No newline at end of file
