@@ -39,6 +39,7 @@ enum t_statechange_notifications
 	STATECHANGE_NEWCONTEXT, /* New context created */
 	STATECHANGE_CHANGEDCONTEXT, /* Currently active context changed */
 	STATECHANGE_REMOVECONTEXT, /* Right before deleting a context */
+	STATECHANGE_GLOBALBOOKMARKS,
 
 	STATECHANGE_MAX
 };
@@ -112,13 +113,13 @@ public:
 	void DestroyEngine();
 
 	CLocalPath GetLocalDir() const;
-	bool SetLocalDir(CLocalPath const& dir, wxString *error = 0, bool rememberPreviousSubdir = true);
-	bool SetLocalDir(const wxString& dir, wxString *error = 0, bool rememberPreviousSubdir = true);
+	bool SetLocalDir(CLocalPath const& dir, std::wstring *error = 0, bool rememberPreviousSubdir = true);
+	bool SetLocalDir(std::wstring const& dir, std::wstring *error = 0, bool rememberPreviousSubdir = true);
 
 	bool Connect(Site const& site, const CServerPath& path = CServerPath(), bool compare = false);
 	bool Disconnect();
 
-	bool ChangeRemoteDir(const CServerPath& path, const wxString& subdir = _T(""), int flags = 0, bool ignore_busy = false, bool compare = false);
+	bool ChangeRemoteDir(CServerPath const& path, std::wstring const& subdir = std::wstring(), int flags = 0, bool ignore_busy = false, bool compare = false);
 	bool SetRemoteDir(std::shared_ptr<CDirectoryListing> const& pDirectoryListing, bool modified = false);
 	std::shared_ptr<CDirectoryListing> GetRemoteDir() const;
 	const CServerPath GetRemotePath() const;
@@ -128,7 +129,7 @@ public:
 	wxString GetTitle() const;
 
 	void RefreshLocal();
-	void RefreshLocalFile(wxString file);
+	void RefreshLocalFile(std::wstring const& file);
 	void LocalDirCreated(const CLocalPath& path);
 
 	bool RefreshRemote();
