@@ -19,7 +19,14 @@ public:
 	bool ShowTool(int id);
 	bool HideTool(int id);
 
+#ifdef __WXMSW__
+	virtual bool Realize();
+#endif
+
 protected:
+	void MakeTool(char const* id, std::wstring const& art, wxString const& tooltip, wxString const& help = wxString(), wxItemKind type = wxITEM_NORMAL);
+	void MakeTools();
+
 	virtual void OnStateChange(CState* pState, t_statechange_notifications notification, const wxString& data, const void* data2);
 	virtual void OnOptionsChanged(changed_options_t const& options);
 
@@ -27,7 +34,10 @@ protected:
 
 	std::map<int, wxToolBarToolBase*> m_hidden_tools;
 
-	DECLARE_DYNAMIC_CLASS(CToolBar)
+#ifdef __WXMSW__
+	std::unique_ptr<wxImageList> toolImages_;
+	std::unique_ptr<wxImageList> disabledToolImages_;
+#endif
 };
 
 #endif //__TOOLBAR_H__
